@@ -5,22 +5,41 @@
 //! The main function is `calculate_cost` which returns &TransactionCost.
 //!
 use {
-    crate::{block_cost_limits::*, execute_cost_table::ExecuteCostTable},
+    crate::{
+        bank::is_simple_vote_transaction, block_cost_limits::*,
+        execute_cost_table::ExecuteCostTable,
+    },
     log::*,
-    solana_sdk::{pubkey::Pubkey, transaction::Transaction},
+    solana_sdk::{pubkey::Pubkey, transaction::SanitizedTransaction},
     std::collections::HashMap,
 };
 
 const MAX_WRITABLE_ACCOUNTS: usize = 256;
 
 // costs are stored in number of 'compute unit's
-#[derive(AbiExample, Default, Debug)]
+#[derive(Debug)]
 pub struct TransactionCost {
     pub writable_accounts: Vec<Pubkey>,
     pub signature_cost: u64,
     pub write_lock_cost: u64,
     pub data_bytes_cost: u64,
     pub execution_cost: u64,
+    // `cost_weight` is a multiplier could be applied to transaction cost,
+    // if set to zero allows the transaction to bypass cost limit check.
+    pub cost_weight: u32,
+}
+
+impl Default for TransactionCost {
+    fn default() -> Self {
+        Self {
+            writable_accounts: Vec::with_capacity(MAX_WRITABLE_ACCOUNTS),
+            signature_cost: 0u64,
+            write_lock_cost: 0u64,
+            data_bytes_cost: 0u64,
+            execution_cost: 0u64,
+            cost_weight: 1u32,
+        }
+    }
 }
 
 impl TransactionCost {
@@ -37,6 +56,7 @@ impl TransactionCost {
         self.write_lock_cost = 0;
         self.data_bytes_cost = 0;
         self.execution_cost = 0;
+        self.cost_weight = 1;
     }
 
     pub fn sum(&self) -> u64 {
@@ -88,7 +108,7 @@ impl CostModel {
 
     pub fn calculate_cost(
         &self,
-        transaction: &Transaction,
+        transaction: &SanitizedTransaction,
         demote_program_write_locks: bool,
     ) -> TransactionCost {
         let mut tx_cost = TransactionCost::new_with_capacity(MAX_WRITABLE_ACCOUNTS);
@@ -97,6 +117,7 @@ impl CostModel {
         self.get_write_lock_cost(&mut tx_cost, transaction, demote_program_write_locks);
         tx_cost.data_bytes_cost = self.get_data_bytes_cost(transaction);
         tx_cost.execution_cost = self.get_transaction_cost(transaction);
+        tx_cost.cost_weight = self.calculate_cost_weight(transaction);
 
         debug!("transaction {:?} has cost {:?}", transaction, tx_cost);
         tx_cost
@@ -119,11 +140,54 @@ impl CostModel {
         self.instruction_execution_cost_table.get_cost_table()
     }
 
-    fn get_signature_cost(&self, transaction: &Transaction) -> u64 {
-        transaction.signatures.len() as u64 * SIGNATURE_COST
+    fn get_signature_cost(&self, transaction: &SanitizedTransaction) -> u64 {
+        transaction.signatures().len() as u64 * SIGNATURE_COST
     }
 
-    pub fn find_instruction_cost(&self, program_key: &Pubkey) -> u64 {
+    fn get_write_lock_cost(
+        &self,
+        tx_cost: &mut TransactionCost,
+        transaction: &SanitizedTransaction,
+        demote_program_write_locks: bool,
+    ) {
+        let message = transaction.message();
+        message.account_keys_iter().enumerate().for_each(|(i, k)| {
+            let is_writable = message.is_writable(i, demote_program_write_locks);
+
+            if is_writable {
+                tx_cost.writable_accounts.push(*k);
+                tx_cost.write_lock_cost += WRITE_LOCK_UNITS;
+            }
+        });
+    }
+
+    fn get_data_bytes_cost(&self, transaction: &SanitizedTransaction) -> u64 {
+        let mut data_bytes_cost: u64 = 0;
+        transaction
+            .message()
+            .program_instructions_iter()
+            .for_each(|(_, ix)| {
+                data_bytes_cost += ix.data.len() as u64 / DATA_BYTES_UNITS;
+            });
+        data_bytes_cost
+    }
+
+    fn get_transaction_cost(&self, transaction: &SanitizedTransaction) -> u64 {
+        let mut cost: u64 = 0;
+
+        for (program_id, instruction) in transaction.message().program_instructions_iter() {
+            let instruction_cost = self.find_instruction_cost(program_id);
+            trace!(
+                "instruction {:?} has cost of {}",
+                instruction,
+                instruction_cost
+            );
+            cost = cost.saturating_add(instruction_cost);
+        }
+        cost
+    }
+
+    fn find_instruction_cost(&self, program_key: &Pubkey) -> u64 {
         match self.instruction_execution_cost_table.get_cost(program_key) {
             Some(cost) => *cost,
             None => {
@@ -137,46 +201,13 @@ impl CostModel {
         }
     }
 
-    fn get_write_lock_cost(
-        &self,
-        tx_cost: &mut TransactionCost,
-        transaction: &Transaction,
-        demote_program_write_locks: bool,
-    ) {
-        let message = transaction.message();
-        message.account_keys.iter().enumerate().for_each(|(i, k)| {
-            let is_writable = message.is_writable(i, demote_program_write_locks);
-
-            if is_writable {
-                tx_cost.writable_accounts.push(*k);
-                tx_cost.write_lock_cost += WRITE_LOCK_UNITS;
-            }
-        });
-    }
-
-    fn get_data_bytes_cost(&self, transaction: &Transaction) -> u64 {
-        let mut data_bytes_cost: u64 = 0;
-        transaction.message().instructions.iter().for_each(|ix| {
-            data_bytes_cost += ix.data.len() as u64 / DATA_BYTES_UNITS;
-        });
-        data_bytes_cost
-    }
-
-    fn get_transaction_cost(&self, transaction: &Transaction) -> u64 {
-        let mut cost: u64 = 0;
-
-        for instruction in &transaction.message().instructions {
-            let program_id =
-                transaction.message().account_keys[instruction.program_id_index as usize];
-            let instruction_cost = self.find_instruction_cost(&program_id);
-            trace!(
-                "instruction {:?} has cost of {}",
-                instruction,
-                instruction_cost
-            );
-            cost = cost.saturating_add(instruction_cost);
+    fn calculate_cost_weight(&self, transaction: &SanitizedTransaction) -> u32 {
+        if is_simple_vote_transaction(transaction) {
+            // vote has zero cost weight, so it bypasses block cost limit checking
+            0u32
+        } else {
+            1u32
         }
-        cost
     }
 }
 
@@ -196,7 +227,9 @@ mod tests {
             signature::{Keypair, Signer},
             system_instruction::{self},
             system_program, system_transaction,
+            transaction::Transaction,
         },
+        solana_vote_program::vote_transaction,
         std::{
             str::FromStr,
             sync::{Arc, RwLock},
@@ -211,7 +244,7 @@ mod tests {
             mint_keypair,
             ..
         } = create_genesis_config(10);
-        let bank = Arc::new(Bank::new_no_wallclock_throttle(&genesis_config));
+        let bank = Arc::new(Bank::new_no_wallclock_throttle_for_tests(&genesis_config));
         let start_hash = bank.last_blockhash();
         (mint_keypair, start_hash)
     }
@@ -244,8 +277,9 @@ mod tests {
         let (mint_keypair, start_hash) = test_setup();
 
         let keypair = Keypair::new();
-        let simple_transaction =
-            system_transaction::transfer(&mint_keypair, &keypair.pubkey(), 2, start_hash);
+        let simple_transaction = SanitizedTransaction::from_transaction_for_tests(
+            system_transaction::transfer(&mint_keypair, &keypair.pubkey(), 2, start_hash),
+        );
         debug!(
             "system_transaction simple_transaction {:?}",
             simple_transaction
@@ -273,7 +307,11 @@ mod tests {
         let instructions =
             system_instruction::transfer_many(&mint_keypair.pubkey(), &[(key1, 1), (key2, 1)]);
         let message = Message::new(&instructions, Some(&mint_keypair.pubkey()));
-        let tx = Transaction::new(&[&mint_keypair], message, start_hash);
+        let tx = SanitizedTransaction::from_transaction_for_tests(Transaction::new(
+            &[&mint_keypair],
+            message,
+            start_hash,
+        ));
         debug!("many transfer transaction {:?}", tx);
 
         // expected cost for two system transfer instructions
@@ -300,12 +338,14 @@ mod tests {
             CompiledInstruction::new(3, &(), vec![0, 1]),
             CompiledInstruction::new(4, &(), vec![0, 2]),
         ];
-        let tx = Transaction::new_with_compiled_instructions(
-            &[&mint_keypair],
-            &[key1, key2],
-            start_hash,
-            vec![prog1, prog2],
-            instructions,
+        let tx = SanitizedTransaction::from_transaction_for_tests(
+            Transaction::new_with_compiled_instructions(
+                &[&mint_keypair],
+                &[key1, key2],
+                start_hash,
+                vec![prog1, prog2],
+                instructions,
+            ),
         );
         debug!("many random transaction {:?}", tx);
 
@@ -330,12 +370,14 @@ mod tests {
             CompiledInstruction::new(4, &(), vec![0, 2]),
             CompiledInstruction::new(5, &(), vec![1, 3]),
         ];
-        let tx = Transaction::new_with_compiled_instructions(
-            &[&signer1, &signer2],
-            &[key1, key2],
-            Hash::new_unique(),
-            vec![prog1, prog2],
-            instructions,
+        let tx = SanitizedTransaction::from_transaction_for_tests(
+            Transaction::new_with_compiled_instructions(
+                &[&signer1, &signer2],
+                &[key1, key2],
+                Hash::new_unique(),
+                vec![prog1, prog2],
+                instructions,
+            ),
         );
 
         let cost_model = CostModel::default();
@@ -369,8 +411,12 @@ mod tests {
     #[test]
     fn test_cost_model_calculate_cost() {
         let (mint_keypair, start_hash) = test_setup();
-        let tx =
-            system_transaction::transfer(&mint_keypair, &Keypair::new().pubkey(), 2, start_hash);
+        let tx = SanitizedTransaction::from_transaction_for_tests(system_transaction::transfer(
+            &mint_keypair,
+            &Keypair::new().pubkey(),
+            2,
+            start_hash,
+        ));
 
         let expected_account_cost = WRITE_LOCK_UNITS * 2;
         let expected_execution_cost = 8;
@@ -383,6 +429,7 @@ mod tests {
         assert_eq!(expected_account_cost, tx_cost.write_lock_cost);
         assert_eq!(expected_execution_cost, tx_cost.execution_cost);
         assert_eq!(2, tx_cost.writable_accounts.len());
+        assert_eq!(1u32, tx_cost.cost_weight);
     }
 
     #[test]
@@ -415,12 +462,14 @@ mod tests {
             CompiledInstruction::new(3, &(), vec![0, 1]),
             CompiledInstruction::new(4, &(), vec![0, 2]),
         ];
-        let tx = Arc::new(Transaction::new_with_compiled_instructions(
-            &[&mint_keypair],
-            &[key1, key2],
-            start_hash,
-            vec![prog1, prog2],
-            instructions,
+        let tx = Arc::new(SanitizedTransaction::from_transaction_for_tests(
+            Transaction::new_with_compiled_instructions(
+                &[&mint_keypair],
+                &[key1, key2],
+                start_hash,
+                vec![prog1, prog2],
+                instructions,
+            ),
         ));
 
         let number_threads = 10;
@@ -487,4 +536,31 @@ mod tests {
             .get_cost(&solana_vote_program::id())
             .is_some());
     }
+
+    #[test]
+    fn test_calculate_cost_weight() {
+        let (mint_keypair, start_hash) = test_setup();
+
+        let keypair = Keypair::new();
+        let simple_transaction = SanitizedTransaction::from_transaction_for_tests(
+            system_transaction::transfer(&mint_keypair, &keypair.pubkey(), 2, start_hash),
+        );
+        let vote_transaction = SanitizedTransaction::from_transaction_for_tests(
+            vote_transaction::new_vote_transaction(
+                vec![42],
+                Hash::default(),
+                Hash::default(),
+                &keypair,
+                &keypair,
+                &keypair,
+                None,
+            ),
+        );
+
+        let testee = CostModel::default();
+
+        // For now, vote has zero weight, everything else is neutral, for now
+        assert_eq!(1u32, testee.calculate_cost_weight(&simple_transaction));
+        assert_eq!(0u32, testee.calculate_cost_weight(&vote_transaction));
+    }
 }
